@@ -336,14 +336,14 @@ class ip4tables(object):
         return table in BUILT_IN_CHAINS and \
                chain in BUILT_IN_CHAINS[table]
 
-    def build_chain(self, add, table, chain):
+    def build_chain_rules(self, add, table, chain):
         rule = [ "-t", table ]
         if add:
             rule.append("-N")
         else:
             rule.append("-X")
         rule.append(chain)
-        return rule
+        return [rule]
 
     def build_rule(self, add, table, chain, index, args):
         rule = [ "-t", table ]
@@ -520,9 +520,9 @@ class ip4tables(object):
         ret = runProg(self._command, ["-w", "-L", "-n"])  # since iptables-1.4.20
         if ret[0] == 0:
             wait_option = "-w"  # wait for xtables lock
-            ret = runProg(self._command, ["-w2", "-L", "-n"])  # since iptables > 1.4.21
+            ret = runProg(self._command, ["-w10", "-L", "-n"])  # since iptables > 1.4.21
             if ret[0] == 0:
-                wait_option = "-w2"  # wait max 2 seconds
+                wait_option = "-w10"  # wait max 10 seconds
             log.debug2("%s: %s will be using %s option.", self.__class__, self._command, wait_option)
 
         return wait_option
@@ -872,7 +872,7 @@ class ip4tables(object):
             rule_fragment += self._rich_rule_destination_fragment(rich_rule.destination)
             rule_fragment += self._rich_rule_source_fragment(rich_rule.source)
         if not rich_rule or rich_rule.action != Rich_Mark:
-            rule_fragment += [ "-m", "conntrack", "--ctstate", "NEW" ]
+            rule_fragment += [ "-m", "conntrack", "--ctstate", "NEW,UNTRACKED" ]
 
         rules = []
         if rich_rule:
@@ -885,17 +885,19 @@ class ip4tables(object):
 
         return rules
 
-    def build_zone_protocol_rules(self, enable, zone, protocol, rich_rule=None):
+    def build_zone_protocol_rules(self, enable, zone, protocol, destination=None, rich_rule=None):
         add_del = { True: "-A", False: "-D" }[enable]
         table = "filter"
         target = DEFAULT_ZONE_TARGET.format(chain=SHORTCUTS["INPUT"], zone=zone)
 
         rule_fragment = [ "-p", protocol ]
+        if destination:
+            rule_fragment += [ "-d", destination ]
         if rich_rule:
             rule_fragment += self._rich_rule_destination_fragment(rich_rule.destination)
             rule_fragment += self._rich_rule_source_fragment(rich_rule.source)
         if not rich_rule or rich_rule.action != Rich_Mark:
-            rule_fragment += [ "-m", "conntrack", "--ctstate", "NEW" ]
+            rule_fragment += [ "-m", "conntrack", "--ctstate", "NEW,UNTRACKED" ]
 
         rules = []
         if rich_rule:
@@ -923,7 +925,7 @@ class ip4tables(object):
             rule_fragment += self._rich_rule_destination_fragment(rich_rule.destination)
             rule_fragment += self._rich_rule_source_fragment(rich_rule.source)
         if not rich_rule or rich_rule.action != Rich_Mark:
-            rule_fragment += [ "-m", "conntrack", "--ctstate", "NEW" ]
+            rule_fragment += [ "-m", "conntrack", "--ctstate", "NEW,UNTRACKED" ]
 
         rules = []
         if rich_rule:
@@ -973,7 +975,7 @@ class ip4tables(object):
 
         rules.append([ add_del, "%s_allow" % (target), "-t", "filter"]
                      + rule_fragment +
-                     ["-m", "conntrack", "--ctstate", "NEW", "-j", "ACCEPT" ])
+                     ["-m", "conntrack", "--ctstate", "NEW,UNTRACKED", "-j", "ACCEPT" ])
 
         return rules
 
@@ -1017,7 +1019,7 @@ class ip4tables(object):
                                             zone=zone)
         rules.append([ add_del, "%s_allow" % (target),
                      "-t", "filter", "-m", "conntrack",
-                     "--ctstate", "NEW" ] +
+                     "--ctstate", "NEW,UNTRACKED" ] +
                      mark + [ "-j", "ACCEPT" ])
 
         return rules
