@@ -425,10 +425,10 @@ class nftables(object):
         default_rules.append("add rule inet %s filter_%s jump filter_%s_ZONES_SOURCE" % (TABLE_NAME, "INPUT", "INPUT"))
         default_rules.append("add rule inet %s filter_%s jump filter_%s_ZONES" % (TABLE_NAME, "INPUT", "INPUT"))
         if log_denied != "off":
-            default_rules.append("add rule inet %s filter_%s ct state invalid log prefix '\"STATE_INVALID_DROP: \"'" % (TABLE_NAME, "INPUT"))
+            default_rules.append("add rule inet %s filter_%s ct state invalid %%%%LOGTYPE%%%% log prefix '\"STATE_INVALID_DROP: \"'" % (TABLE_NAME, "INPUT"))
         default_rules.append("add rule inet %s filter_%s ct state invalid drop" % (TABLE_NAME, "INPUT"))
         if log_denied != "off":
-            default_rules.append("add rule inet %s filter_%s log prefix '\"FINAL_REJECT: \"'" % (TABLE_NAME, "INPUT"))
+            default_rules.append("add rule inet %s filter_%s %%%%LOGTYPE%%%% log prefix '\"FINAL_REJECT: \"'" % (TABLE_NAME, "INPUT"))
         default_rules.append("add rule inet %s filter_%s reject with icmpx type admin-prohibited" % (TABLE_NAME, "INPUT"))
 
         # filter, FORWARD
@@ -443,10 +443,10 @@ class nftables(object):
         default_rules.append("add rule inet %s filter_%s jump filter_%s_OUT_ZONES_SOURCE" % (TABLE_NAME, "FORWARD", "FORWARD"))
         default_rules.append("add rule inet %s filter_%s jump filter_%s_OUT_ZONES" % (TABLE_NAME, "FORWARD", "FORWARD"))
         if log_denied != "off":
-            default_rules.append("add rule inet %s filter_%s ct state invalid log prefix '\"STATE_INVALID_DROP: \"'" % (TABLE_NAME, "FORWARD"))
+            default_rules.append("add rule inet %s filter_%s ct state invalid %%%%LOGTYPE%%%% log prefix '\"STATE_INVALID_DROP: \"'" % (TABLE_NAME, "FORWARD"))
         default_rules.append("add rule inet %s filter_%s ct state invalid drop" % (TABLE_NAME, "FORWARD"))
         if log_denied != "off":
-            default_rules.append("add rule inet %s filter_%s log prefix '\"FINAL_REJECT: \"'" % (TABLE_NAME, "FORWARD"))
+            default_rules.append("add rule inet %s filter_%s %%%%LOGTYPE%%%% log prefix '\"FINAL_REJECT: \"'" % (TABLE_NAME, "FORWARD"))
         default_rules.append("add rule inet %s filter_%s reject with icmpx type admin-prohibited" % (TABLE_NAME, "FORWARD"))
 
         OUR_CHAINS["inet"]["filter"] = set(["INPUT_ZONES_SOURCE",
@@ -700,7 +700,7 @@ class nftables(object):
         rule = [add_del, "rule", "inet", "%s" % TABLE_NAME,
                 "%s_%s_allow" % (table, target)]
         rule += rule_fragment + ["log", "level", "audit"]
-        rule += self._rich_rule_limit_fragment(rich_rule.log.limit)
+        rule += self._rich_rule_limit_fragment(rich_rule.audit.limit)
 
         return rule
 
@@ -812,6 +812,7 @@ class nftables(object):
             rule_fragment += self._rich_rule_destination_fragment(rich_rule.destination)
             rule_fragment += self._rich_rule_source_fragment(rich_rule.source)
         rule_fragment += [proto, "dport", "%s" % portStr(port, "-")]
+        rule_fragment += ["ct", "state", "new,untracked"]
 
         rules = []
         if rich_rule:
@@ -821,21 +822,30 @@ class nftables(object):
         else:
             rules.append([add_del, "rule", "inet", "%s" % TABLE_NAME,
                           "%s_%s_allow" % (table, target)] +
-                          rule_fragment + ["ct", "state", "new", "accept"])
+                          rule_fragment + ["accept"])
 
         return rules
 
-    def build_zone_protocol_rules(self, enable, zone, protocol, rich_rule=None):
+    def build_zone_protocol_rules(self, enable, zone, protocol, destination=None, rich_rule=None):
         add_del = { True: "add", False: "delete" }[enable]
         table = "filter"
         target = DEFAULT_ZONE_TARGET.format(chain=SHORTCUTS["INPUT"], zone=zone)
 
         rule_fragment = []
+        if rich_rule:
+            rule_fragment += self._rich_rule_family_fragment(rich_rule.family)
+        if destination:
+            if check_single_address("ipv4", destination):
+                rule_fragment += ["ip"]
+            else:
+                rule_fragment += ["ip6"]
+            rule_fragment += ["daddr", destination]
         if rich_rule:
             rule_fragment += self._rich_rule_family_fragment(rich_rule.family)
             rule_fragment += self._rich_rule_destination_fragment(rich_rule.destination)
             rule_fragment += self._rich_rule_source_fragment(rich_rule.source)
         rule_fragment = ["meta", "l4proto", protocol]
+        rule_fragment += ["ct", "state", "new,untracked"]
 
         rules = []
         if rich_rule:
@@ -845,7 +855,7 @@ class nftables(object):
         else:
             rules.append([add_del, "rule", "inet", "%s" % TABLE_NAME,
                           "filter_%s_allow" % (target)] +
-                          rule_fragment + ["ct", "state", "new", "accept"])
+                          rule_fragment + ["accept"])
 
         return rules
 
@@ -868,6 +878,7 @@ class nftables(object):
             rule_fragment += self._rich_rule_destination_fragment(rich_rule.destination)
             rule_fragment += self._rich_rule_source_fragment(rich_rule.source)
         rule_fragment += [proto, "sport", "%s" % portStr(port, "-")]
+        rule_fragment += ["ct", "state", "new,untracked"]
 
         rules = []
         if rich_rule:
@@ -877,7 +888,7 @@ class nftables(object):
         else:
             rules.append([add_del, "rule", "inet", "%s" % TABLE_NAME,
                           "%s_%s_allow" % (table, target)] +
-                          rule_fragment + ["ct", "state", "new", "accept"])
+                          rule_fragment + ["accept"])
 
         return rules
 
@@ -938,7 +949,7 @@ class nftables(object):
 
         rules.append([add_del, "rule", "inet", "%s" % TABLE_NAME,
                       "filter_%s_allow" % (target)]
-                      + rule_fragment + ["ct", "state", "new", "accept"])
+                      + rule_fragment + ["ct", "state", "new,untracked", "accept"])
 
         return rules
 
@@ -1002,7 +1013,7 @@ class nftables(object):
         target = DEFAULT_ZONE_TARGET.format(chain=SHORTCUTS[filter_chain],
                                             zone=zone)
         rules.append([add_del, "rule", "inet", "%s" % TABLE_NAME,
-                      "filter_%s_allow" % (target), "ct", "state", "new"]
+                      "filter_%s_allow" % (target), "ct", "state", "new,untracked"]
                       + mark_fragment + ["accept"])
 
         return rules
@@ -1060,8 +1071,9 @@ class nftables(object):
                 else:
                     if self._fw.get_log_denied() != "off" and final_target != "accept":
                         rules.append([add_del, "rule", "inet", "%s" % TABLE_NAME,
-                                      final_chain] + rule_fragment + ["log",
-                                      "prefix", "\"%s_%s_ICMP_BLOCK: \"" % (table, zone)])
+                                      final_chain] + rule_fragment +
+                                     ["%%LOGTYPE%%", "log", "prefix",
+                                      "\"%s_%s_ICMP_BLOCK: \"" % (table, zone)])
                     rules.append([add_del, "rule", "inet", "%s" % TABLE_NAME,
                                   final_chain] + rule_fragment + [final_target])
 
